@@ -1,6 +1,6 @@
 ;;;; remote-rep.jl -- Remote file access via the rep-remote program
 ;;;  Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
-;;;  $Id: rep.jl,v 1.16 2000/09/11 20:23:12 john Exp $
+;;;  $Id: rep.jl,v 1.17 2001/03/26 21:51:12 jsh Exp $
 
 ;;; This file is part of librep.
 
@@ -202,7 +202,8 @@
     (error "rep-remote session is dying"))
   (while (remote-rep-status-p session status)
     (when (and (process-running-p (aref session remote-rep-process))
-	       (accept-process-output remote-rep-timeout))
+	       (accept-process-output-1 (aref session remote-rep-process)
+					remote-rep-timeout))
       (aset session remote-rep-status 'timed-out)
       (error "rep-remote process timed out (%s)" (or type "unknown")))))
 
@@ -297,8 +298,9 @@
 			 (substring output point))
 		   (setq point (length output)))))
 	      (t
-	       (format standard-error "remote-rep: unhandled output %S\n"
-		       (substring output point))
+	       (unless (string-looking-at "\\s*$" output point)
+		 (format standard-error "remote-rep: unhandled output %S\n"
+			 (substring output point)))
 	       (setq point (length output))))))))
 
 (defun remote-rep-sentinel (process)
