@@ -1,6 +1,6 @@
 #| rep.jl -- inliners for many rep language features
 
-   $Id: rep.jl,v 1.41 2000/09/12 10:32:17 john Exp $
+   $Id: rep.jl,v 1.44 2001/06/15 04:06:55 jsh Exp $
 
    Copyright (C) 2000 John Harper <john@dcs.warwick.ac.uk>
 
@@ -264,9 +264,13 @@
 	 (value (nth 2 form))
 	 (doc (nth 3 form)))
       (remember-variable name)
+      (when (and (compiler-constant-p doc)
+		 (stringp (compiler-constant-value doc))
+		 *compiler-write-docs*)
+	(add-documentation name nil (compiler-constant-value doc))
+	(setq doc nil))
       `(progn
-	 (when ,doc
-	   (put ',name 'variable-documentation ,doc))
+	 ,@(and doc (list `(put ',name 'variable-documentation ,doc)))
 	 (make-variable-special ',name)
 	 (unless (boundp ',name)
 	   (setq ,name ,value)))))
@@ -554,16 +558,17 @@
 		(compile-form-1 (car subl) return-follows)
 		(decrement-stack)))
 	    (when (consp (cdr form))
-	      (compiler-warning
-	       'misc "Unreachable conditions after t in cond statement")
+	      ;;(compiler-warning
+	      ;; 'misc "Unreachable conditions after t in cond statement")
 	      ;; Ignore the rest of the statement
 	      (setq form nil))
 	    (setq need-trailing-nil nil))
 	   ((eq condition nil)
 	    ;; condition nil -- never taken
 	    (when (cdr subl)
-	      (compiler-warning
-	       'misc "Unreachable forms after nil in cond statement")))
+	      ;;(compiler-warning
+	      ;; 'misc "Unreachable forms after nil in cond statement")
+	      ))
 	   (t
 	    ;; non t-or-nil condition
 	    (compile-form-1 (car subl) (and return-follows
