@@ -1,6 +1,6 @@
 /* files.c -- Extendable file handling
    Copyright (C) 1998 John Harper <john@dcs.warwick.ac.uk>
-   $Id: files.c,v 1.27 2000/07/14 11:08:37 john Exp $
+   $Id: files.c,v 1.31 2000/09/03 12:36:04 john Exp $
 
    This file is part of Jade.
 
@@ -97,9 +97,6 @@ Buffer-local variable absolutely defining the directory to which all files
 accessed in the buffer are resolved from (unless they're absolute.) 
 ::end:: */
 
-/* file-handler environment, (SYMBOL . FUNCTION) or Qt */
-repv rep_fh_env;
-
 /* List of all allocated file objects */
 static rep_file *file_list;
 
@@ -159,6 +156,8 @@ int rep_op_insert_file_contents = op_insert_file_contents;
 
 
 
+DEFSYM (rep_io_file_handlers, "rep.io.file-handlers");
+
 static inline repv
 get_fh_env (void)
 {
@@ -166,6 +165,20 @@ get_fh_env (void)
     return rep_VOIDP (ret) ? Qt : ret;
 }
 
+/* this is duplicated in rep/io/file-handlers.jl */
+static inline repv
+file_handler_ref (repv handler)
+{
+    repv tem = Fget_structure (Qrep_io_file_handlers);
+    if (tem != Qnil)
+    {
+	tem = F_structure_ref (tem, handler);
+	if (!tem || rep_VOIDP (tem))
+	    tem = Qnil;
+    }
+    return tem;
+}
+
 repv
 rep_signal_file_error(repv cdr)
 {
@@ -238,7 +251,6 @@ rep_call_file_handler(repv handler, int op, repv sym, int nargs, ...)
     repv res;
     int i;
     va_list args;
-    struct rep_Call lc;
 
     va_start(args, nargs);
     for(i = 0; i < nargs; i++)
@@ -249,11 +261,6 @@ rep_call_file_handler(repv handler, int op, repv sym, int nargs, ...)
     va_end(args);
     arg_list = Fcons(sym, arg_list);
 
-    lc.fun = Qnil;
-    lc.args = Qnil;
-    lc.args_evalled_p = Qnil;
-    rep_PUSH_CALL(lc);
-
     /* before it gets dereferenced */
     op_data.handler = handler;
 
@@ -261,20 +268,14 @@ rep_call_file_handler(repv handler, int op, repv sym, int nargs, ...)
     {
 	repv fh_env = get_fh_env ();
 	if (fh_env == Qt)
-	{
-	    rep_USE_DEFAULT_ENV;
-	    handler = Fsymbol_value (handler, Qt);
-	}
+	    handler = file_handler_ref (handler);
 	else
 	{
 	    repv tem = Fassq (handler, fh_env);
 	    if (tem && rep_CONSP(tem))
 	    {
 		if (rep_CDR(tem) == Qt)
-		{
-		    rep_USE_DEFAULT_ENV;
-		    handler = Fsymbol_value (handler, Qt);
-		}
+		    handler = file_handler_ref (handler);
 		else if (rep_FUNARGP(rep_CDR(tem)))
 		    handler = rep_CDR(tem);
 	    }
@@ -294,7 +295,6 @@ rep_call_file_handler(repv handler, int op, repv sym, int nargs, ...)
     else
 	res = rep_NULL;
 
-    rep_POP_CALL(lc);
     return res;
 }
 
@@ -370,7 +370,7 @@ rep_localise_and_get_handler(repv *file_namep, int op)
 
 DEFUN("file-name-absolute-p", Ffile_name_absolute_p,
       Sfile_name_absolute_p, (repv file), rep_Subr1) /*
-::doc:file-name-absolute-p::
+::doc:rep.io.files#file-name-absolute-p::
 file-name-absolute-p FILE-NAME
 
 Returns t if FILE-NAME is context-independent, i.e. it does not name a file
@@ -389,7 +389,7 @@ relative to the default-directory.
 
 DEFUN("expand-file-name", Fexpand_file_name, Sexpand_file_name,
       (repv file_name, repv dir_name), rep_Subr2) /*
-::doc:expand-file-name::
+::doc:rep.io.files#expand-file-name::
 expand-file-name FILE-NAME [BASE-DIR]
 
 Expands FILE-NAME assuming that it specifies a file relative to BASE-DIR.
@@ -451,7 +451,7 @@ or whatever).
 
 DEFUN("local-file-name", Flocal_file_name, Slocal_file_name,
       (repv file), rep_Subr1) /*
-::doc:local-file-name::
+::doc:rep.io.files#local-file-name::
 local-file-name FILE-NAME
 
 When possible, return a string absolutely naming the file in the local
@@ -472,7 +472,7 @@ a file in the local system, return nil.
 
 DEFUN("canonical-file-name", Fcanonical_file_name, Scanonical_file_name,
       (repv file), rep_Subr1) /*
-::doc:canonical-file-name::
+::doc:rep.io.files#canonical-file-name::
 canonical-file-name FILE-NAME
 
 Return the canonical name of the file called FILE-NAME. The canonical name
@@ -496,7 +496,7 @@ operating systems, symbolic links will be expanded where possible.)
 
 DEFUN("file-name-nondirectory", Ffile_name_nondirectory,
       Sfile_name_nondirectory, (repv file), rep_Subr1) /*
-::doc:file-name-nondirectory::
+::doc:rep.io.files#file-name-nondirectory::
 file-name-nondirectory FILE-NAME
 
 Return the directory component of FILE-NAME, including the final
@@ -515,7 +515,7 @@ directory separator.
 
 DEFUN("file-name-directory", Ffile_name_directory,
       Sfile_name_directory, (repv file), rep_Subr1) /*
-::doc:file-name-directory::
+::doc:rep.io.files#file-name-directory::
 file-name-directory FILE-NAME
 
 Return the file name component of FILE-NAME, i.e. everything following
@@ -534,7 +534,7 @@ the final directory separator.
 
 DEFUN("file-name-as-directory", Ffile_name_as_directory,
       Sfile_name_as_directory, (repv file), rep_Subr1) /*
-::doc:file-name-as-directory::
+::doc:rep.io.files#file-name-as-directory::
 file-name-as-directory FILE-NAME
 
 Return FILE-NAME such that it names a directory (i.e with a terminating
@@ -553,7 +553,7 @@ directory separator character.)
 
 DEFUN("directory-file-name", Fdirectory_file_name,
       Sdirectory_file_name, (repv file), rep_Subr1) /*
-::doc:directory-file-name::
+::doc:rep.io.files#directory-file-name::
 directory-file-name DIR-NAME
 
 Return the name of the file representing the directory called DIR-NAME.
@@ -598,7 +598,7 @@ input_handler_callback (int fd)
 
 DEFUN("set-input-handler", Fset_input_handler, Sset_input_handler,
       (repv file, repv function), rep_Subr2) /*
-::doc:set-input-handler::
+::doc:rep.io.files#set-input-handler::
 set-input-handler LOCAL-FILE FUNCTION
 
 Arrange for FUNCTION to be called whenever pending input is available
@@ -725,7 +725,7 @@ file_mark(repv val)
 }
 
 DEFUN("filep", Ffilep, Sfilep, (repv arg), rep_Subr1) /*
-::doc:filep::
+::doc:rep.io.files#filep::
 filep ARG
 
 Returns t if ARG is a file object.
@@ -736,7 +736,7 @@ Returns t if ARG is a file object.
 
 DEFUN("file-binding", Ffile_binding, Sfile_binding,
       (repv file), rep_Subr1) /*
-::doc:file-binding::
+::doc:rep.io.files#file-binding::
 file-binding FILE
 
 Returns the name of the logical file that FILE was opened to access, or nil
@@ -749,7 +749,7 @@ if it has been closed, but is still to be garbage collected.
 
 DEFUN("file-bound-stream", Ffile_bound_stream, Sfile_bound_stream,
       (repv file), rep_Subr1) /*
-::doc:file-bound-stream::
+::doc:rep.io.files#file-bound-stream::
 file-bound-stream FILE
 
 If file object FILE doesn't refer to a local file, return the stream
@@ -762,7 +762,7 @@ that it's bound to.
 
 DEFUN("file-handler-data", Ffile_handler_data, Sfile_handler_data,
       (repv file), rep_Subr1) /*
-::doc:file-handler-data::
+::doc:rep.io.files#file-handler-data::
 file-handler-data FILE
 
 Return the handler-specific data for FILE.
@@ -774,7 +774,7 @@ Return the handler-specific data for FILE.
 
 DEFUN("set-file-handler-data", Fset_file_handler_data,
       Sset_file_handler_data, (repv file, repv data), rep_Subr2) /*
-::doc:set-file-handler-data::
+::doc:rep.io.files#set-file-handler-data::
 set-file-handler-data FILE DATA
 
 Set the handler-specific data of file object FILE to DATA.
@@ -790,7 +790,7 @@ Set the handler-specific data of file object FILE to DATA.
 
 DEFUN("open-file", Fopen_file, Sopen_file,
       (repv file_name, repv access_type), rep_Subr2) /*
-::doc:open-file::
+::doc:rep.io.files#open-file::
 open-file FILE-NAME ACCESS-TYPE
 
 Return a new file object referencing the logical file called FILE-NAME,
@@ -844,7 +844,7 @@ DEFUN("make-file-from-stream", Fmake_file_from_stream,
       Smake_file_from_stream,
       (repv file_name, repv stream, repv handler),
       rep_Subr3) /*
-::doc:make-file-from-stream::
+::doc:rep.io.files#make-file-from-stream::
 make-file-from-stream FILE-NAME STREAM HANDLER
 
 Return a new file object that refers to the logical file called FILE-NAME,
@@ -866,7 +866,7 @@ function HANDLER.
 }
 
 DEFUN("close-file", Fclose_file, Sclose_file, (repv file), rep_Subr1) /*
-::doc:close-file::
+::doc:rep.io.files#close-file::
 close-file FILE
 
 Signal that there will be no more I/O through the file object FILE.
@@ -903,7 +903,7 @@ Signal that there will be no more I/O through the file object FILE.
 
 DEFUN("flush-file", Fflush_file, Sflush_file,
       (repv file), rep_Subr1) /*
-::doc:flush-file::
+::doc:rep.io.files#flush-file::
 flush-file FILE
 
 Flush any buffered output on FILE. This is usually unnecessary since
@@ -923,7 +923,7 @@ all output will be flushed when FILE is eventually closed.
 
 DEFUN("seek-file", Fseek_file, Sseek_file,
       (repv file, repv offset, repv where), rep_Subr3) /*
-::doc:seek-file::
+::doc:rep.io.files#seek-file::
 seek-file FILE [OFFSET] [WHERE-FROM]
 
 Called as (seek-file FILE), returns the distance in bytes from the start
@@ -971,7 +971,7 @@ current position will also fail.
 
 DEFUN_INT("delete-file", Fdelete_file, Sdelete_file, (repv file_name),
 	  rep_Subr1, "fFile to delete:") /*
-::doc:delete-file::
+::doc:rep.io.files#delete-file::
 delete-file FILE-NAME
 
 Delete the file called FILE-NAME.
@@ -991,7 +991,7 @@ DEFSTRING(cant_rename, "Can't rename files across handlers");
 DEFUN_INT("rename-file", Frename_file, Srename_file,
 	  (repv old, repv new), rep_Subr2,
 	  "fOld name of file:" rep_DS_NL "FNew name of file:") /*
-::doc:rename-file::
+::doc:rep.io.files#rename-file::
 rename-file OLD-NAME NEW-NAME
 
 Rename the file called OLD-NAME so that it is called NEW-NAME. Note that
@@ -1026,7 +1026,7 @@ this almost certainly won't work across filing systems.
 DEFUN_INT("make-directory", Fmake_directory, Smake_directory,
 	  (repv dir_name), rep_Subr1,
 	  "DDirectory to create:") /*
-::doc:make-directory::
+::doc:rep.io.files#make-directory::
 make-directory DIRECTORY-NAME
 
 Create a directory called DIRECTORY-NAME.
@@ -1045,7 +1045,7 @@ Create a directory called DIRECTORY-NAME.
 DEFUN_INT("delete-directory", Fdelete_directory, Sdelete_directory,
 	  (repv dir_name), rep_Subr1,
 	  "DDirectory to delete:") /*
-::doc:delete-directory::
+::doc:rep.io.files#delete-directory::
 delete-directory DIRECTORY-NAME
 
 Delete the directory called DIRECTORY-NAME. Note that the directory in
@@ -1064,7 +1064,7 @@ question should be empty.
 
 DEFUN_INT("copy-file", Fcopy_file, Scopy_file, (repv src, repv dst),
 	  rep_Subr2, "fSource file:" rep_DS_NL "FDestination file:") /*
-::doc:copy-file::
+::doc:rep.io.files#copy-file::
 copy-file SOURCE DESTINATION
 
 Create a new copy of the file called SOURCE, as the file called DESTINATION.
@@ -1130,7 +1130,7 @@ Create a new copy of the file called SOURCE, as the file called DESTINATION.
 
 DEFUN("file-readable-p", Ffile_readable_p, Sfile_readable_p,
       (repv file), rep_Subr1) /*
-::doc:file-readable-p::
+::doc:rep.io.files#file-readable-p::
 file-readable-p FILE-NAME
 
 Returns t if the file called FILE-NAME is available for reading from.
@@ -1148,7 +1148,7 @@ Returns t if the file called FILE-NAME is available for reading from.
 
 DEFUN("file-writable-p", Ffile_writable_p, Sfile_writable_p,
       (repv file), rep_Subr1) /*
-::doc:file-writeable-p::
+::doc:rep.io.files#file-writeable-p::
 file-writable-p FILE-NAME
 
 Returns t if the file called FILE-NAME is available for writing to.
@@ -1166,7 +1166,7 @@ Returns t if the file called FILE-NAME is available for writing to.
 
 DEFUN("file-exists-p", Ffile_exists_p, Sfile_exists_p,
       (repv file), rep_Subr1) /*
-::doc:file-exists-p::
+::doc:rep.io.files#file-exists-p::
 file-exists-p FILE-NAME
 
 Returns t if the file called FILE-NAME exists.
@@ -1184,7 +1184,7 @@ Returns t if the file called FILE-NAME exists.
 
 DEFUN("file-regular-p", Ffile_regular_p, Sfile_regular_p,
       (repv file), rep_Subr1) /*
-::doc:file-regular-p::
+::doc:rep.io.files#file-regular-p::
 file-regular-p FILE-NAME
 
 Returns t if the file called FILE-NAME is a normal file, ie, not a
@@ -1203,7 +1203,7 @@ directory, device, symbolic link, etc...
 
 DEFUN("file-directory-p", Ffile_directory_p, Sfile_directory_p,
       (repv file), rep_Subr1) /*
-::doc:file-directory-p::
+::doc:rep.io.files#file-directory-p::
 file-directory-p FILE-NAME
 
 Returns t if the file called FILE-NAME is a directory.
@@ -1221,7 +1221,7 @@ Returns t if the file called FILE-NAME is a directory.
 
 DEFUN("file-symlink-p", Ffile_symlink_p, Sfile_symlink_p,
       (repv file), rep_Subr1) /*
-::doc:file-symlink-p::
+::doc:rep.io.files#file-symlink-p::
 file-symlink-p FILE-NAME
 
 Returns t if the file called FILE-NAME is a symbolic link to another file.
@@ -1239,7 +1239,7 @@ Returns t if the file called FILE-NAME is a symbolic link to another file.
 
 DEFUN("file-owner-p", Ffile_owner_p, Sfile_owner_p,
       (repv file), rep_Subr1) /*
-::doc:file-owner-p::
+::doc:rep.io.files#file-owner-p::
 file-owner-p FILE-NAME
 
 Returns t if the ownership (uid & gid) of the file called FILE-NAME is the
@@ -1258,7 +1258,7 @@ same as that of any files written by the editor.
 
 DEFUN("file-nlinks", Ffile_nlinks, Sfile_nlinks,
       (repv file), rep_Subr1) /*
-::doc:file-nlinks::
+::doc:rep.io.files#file-nlinks::
 file-nlinks FILE-NAME
 
 Returns the number of links pointing to the file called FILE-NAME. This will
@@ -1277,7 +1277,7 @@ be one if FILE-NAME has only one name. Doesn't count symbolic links.
 
 DEFUN("file-size", Ffile_size, Sfile_size,
       (repv file), rep_Subr1) /*
-::doc:file-size::
+::doc:rep.io.files#file-size::
 file-size FILE-NAME
 
 Returns the size of the file called FILE-NAME in bytes.
@@ -1295,7 +1295,7 @@ Returns the size of the file called FILE-NAME in bytes.
 
 DEFUN("file-modes", Ffile_modes, Sfile_modes,
       (repv file), rep_Subr1) /*
-::doc:file-modes::
+::doc:rep.io.files#file-modes::
 file-modes FILE-NAME
 
 Return the access permissions of the file called FILE-NAME. Note that the
@@ -1315,7 +1315,7 @@ is as an argument to set-file-modes.
 
 DEFUN("set-file-modes", Fset_file_modes, Sset_file_modes,
       (repv file, repv modes), rep_Subr2) /*
-::doc:set-file-modes::
+::doc:rep.io.files#set-file-modes::
 set-file-modes FILE-NAME MODES
 
 Sets the access permissions of the file called FILE-NAME to MODES. The only
@@ -1335,7 +1335,7 @@ may change across filing systems.
 
 DEFUN("file-modes-as-string", Ffile_modes_as_string,
       Sfile_modes_as_string, (repv file), rep_Subr1) /*
-::doc:file-modes-as-string::
+::doc:rep.io.files#file-modes-as-string::
 file-modes-as-string FILE-NAME
 
 Returns a ten character string describing the attributes of the file
@@ -1354,7 +1354,7 @@ called FILE-NAME.
 
 DEFUN("file-modtime", Ffile_modtime, Sfile_modtime,
       (repv file), rep_Subr1) /*
-::doc:file-modtime::
+::doc:rep.io.files#file-modtime::
 file-modtime FILE-NAME
 
 Return the time that the file called FILE-NAME was last modified, as a cons
@@ -1404,7 +1404,7 @@ rep_file_newer_than(repv name1, repv name2)
 
 DEFUN("directory-files", Fdirectory_files, Sdirectory_files,
       (repv dir), rep_Subr1) /*
-::doc:directory-files::
+::doc:rep.io.files#directory-files::
 directory-files DIRECTORY
 
 Returns a list of the names of all files in the directory called DIRECTORY.
@@ -1422,7 +1422,7 @@ The list is unsorted.
 }
 
 DEFUN("read-symlink", Fread_symlink, Sread_symlink, (repv file), rep_Subr1) /*
-::doc:read-symlink::
+::doc:rep.io.files#read-symlink::
 read-symlink FILENAME
 
 Return the string that is the contents of the symbolic link FILENAME. This
@@ -1443,7 +1443,7 @@ Signals an error if FILENAME isn't a symbolic link.
 
 DEFUN("make-symlink", Fmake_symlink, Smake_symlink,
       (repv file, repv contents), rep_Subr2) /*
-::doc:make-symlink::
+::doc:rep.io.files#make-symlink::
 make-symlink FILENAME CONTENTS
 
 Create a symbolic link FILENAME pointing to the file called CONTENTS.
@@ -1484,7 +1484,7 @@ rep_file_fdopen (int fd, char *mode)
 
 DEFSTRING(stdin_name, "<stdin>");
 DEFUN("stdin-file", Fstdin_file, Sstdin_file, (void), rep_Subr0) /*
-::doc:stdin-file::
+::doc:rep.io.files#stdin-file::
 stdin-file
 
 Returns the file object representing the editor's standard input.
@@ -1504,7 +1504,7 @@ Returns the file object representing the editor's standard input.
 
 DEFSTRING(stdout_name, "<stdout>");
 DEFUN("stdout-file", Fstdout_file, Sstdout_file, (void), rep_Subr0) /*
-::doc:stdout-file::
+::doc:rep.io.files#stdout-file::
 stdout-file
 
 Returns the file object representing the editor's standard output.
@@ -1524,7 +1524,7 @@ Returns the file object representing the editor's standard output.
 
 DEFSTRING(stderr_name, "<stderr>");
 DEFUN("stderr-file", Fstderr_file, Sstderr_file, (void), rep_Subr0) /*
-::doc:stderr-file::
+::doc:rep.io.files#stderr-file::
 stderr-file
 
 Returns the file object representing the editor's standard output.
@@ -1544,7 +1544,7 @@ Returns the file object representing the editor's standard output.
 
 DEFSTRING(no_temp, "Can't create temporary file name");
 DEFUN("make-temp-name", Fmake_temp_name, Smake_temp_name, (void), rep_Subr0) /*
-::doc:make-temp-name::
+::doc:rep.io.files#make-temp-name::
 make-temp-name
 
 Returns the name of a unique file in the local filing system.
@@ -1559,11 +1559,11 @@ Returns the name of a unique file in the local filing system.
 
 DEFUN("set-file-handler-environment", Fset_file_handler_environment,
       Sset_file_handler_environment, (repv env, repv structure), rep_Subr2) /*
-::doc:set-file-handler-environment::
+::doc:rep.io.files#set-file-handler-environment::
 set-file-handler-environment ENV
 ::end:: */
 {
-    return F_structure_set (structure, Qfh_env_key, env);
+    return Fstructure_define (structure, Qfh_env_key, env);
 }
 
 
@@ -1625,6 +1625,10 @@ rep_files_init(void)
     rep_INTERN(start); rep_INTERN(end);
     rep_INTERN(read); rep_INTERN(write); rep_INTERN(append);
 
+    rep_INTERN(rep_io_file_handlers);
+
+    tem = rep_push_structure ("rep.io.files");
+
     rep_ADD_SUBR(Sfilep);
     rep_ADD_SUBR(Sfile_binding);
     rep_ADD_SUBR(Sfile_bound_stream);
@@ -1677,14 +1681,13 @@ rep_files_init(void)
     rep_ADD_SUBR(Smake_temp_name);
     rep_ADD_SUBR(Sset_file_handler_environment);
 
+    rep_pop_structure (tem);
+
     /* Initialise the type information. */
     rep_file_type = rep_register_new_type("file", rep_ptr_cmp,
 					  file_prin, file_prin, file_sweep,
 					  file_mark, mark_input_handlers,
 					  0, 0, 0, 0, 0, 0);
-
-    rep_fh_env = Qt;
-    rep_mark_static (&rep_fh_env);
 }
 
 void
