@@ -1,6 +1,6 @@
 /* numbers.c -- Implement the tower of numeric types
    Copyright (C) 1993, 1994, 2000 John Harper <john@dcs.warwick.ac.uk>
-   $Id: numbers.c,v 1.46.2.1 2001/04/09 21:38:52 jsh Exp $
+   $Id: numbers.c,v 1.48 2001/06/15 05:24:50 jsh Exp $
 
    This file is part of librep.
 
@@ -975,7 +975,10 @@ rep_parse_number (char *buf, u_int len, u_int radix, int sign, u_int type)
 	{
 	    z = make_number (rep_NUMBER_BIGNUM);
 #ifdef HAVE_GMP
-	    if (mpz_init_set_str (z->z, buf, radix) == 0)
+	    copy = alloca (len + 1);
+	    memcpy (copy, buf, len);
+	    copy[len] = 0;
+	    if (mpz_init_set_str (z->z, copy, radix) == 0)
 	    {
 		if (sign < 0)
 		    mpz_neg (z->z, z->z);
