@@ -1,5 +1,5 @@
 /* repgdbm.c -- rep wrapper to libgdbm
-   $Id: repgdbm.c,v 1.7 2000/09/03 12:36:04 john Exp $ */
+   $Id: repgdbm.c,v 1.8 2001/06/18 17:58:09 jsh Exp $ */
 
 #define _GNU_SOURCE
 
@@ -261,3 +261,14 @@ rep_dl_init (void)
     rep_ADD_SUBR(Sgdbmp);
     return rep_pop_structure (tem);
 }
+
+void
+rep_dl_kill (void)
+{
+    rep_dbm *db;
+    for (db = dbm_chain; db != 0; db = db->next)
+    {
+	if (db->dbm != 0)
+	    Fgdbm_close (rep_VAL (db));
+    }
+}
