@@ -1,6 +1,6 @@
 /* repint.h -- Main include file for library internal objects
    Copyright (C) 1993, 1994 John Harper <john@dcs.warwick.ac.uk>
-   $Id: repint.h,v 1.13 2000/07/15 11:40:55 john Exp $
+   $Id: repint.h,v 1.19 2000/07/31 13:41:55 john Exp $
 
    This file is part of Jade.
 
@@ -31,8 +31,15 @@
 #define POS(x)   MAX(x, 0)
 #define ABS(x)   MAX(x, -(x))
 
+#define rep_INTERNAL 1
 #include "rep.h"
 
+#ifndef ENABLE_BROKEN_DUMPING
+  /* No point incurring the overhead if it's unnecessary */
+# undef rep_CONS_WRITABLE_P
+# define rep_CONS_WRITABLE_P(x) rep_TRUE
+#endif
+
 #ifdef rep_HAVE_UNIX
 # include "unix_defs.h"
 #else
@@ -97,8 +104,8 @@ struct rep_struct_node_struct {
     rep_struct_node *next;
     repv symbol;
     repv binding;
-    int is_constant : 1;
-    int is_exported : 1;
+    u_int is_constant : 1;
+    u_int is_exported : 1;
 };
 
 /* structure encapsulating a single namespace */
@@ -117,7 +124,8 @@ struct rep_struct_struct {
        environment, or Qt to denote all specials. */
     repv special_env;
 
-    /* Bytecode interpreter to use when calling functions defined here */
+    /* Bytecode interpreter to use when calling functions defined here.
+       If null, call rep_apply_bytecode  */
     repv (*apply_bytecode) (repv subr, int nargs, repv *args);
 };
 
@@ -132,9 +140,12 @@ extern int rep_structure_type;
 /* If set, all (local) bindings are exported by default. */
 #define rep_STF_EXPORT_ALL	(1 << (rep_CELL16_TYPE_BITS + 1))
 
+/* If set, bindings can be created by setq et al. */
+#define rep_STF_SET_BINDS	(1 << (rep_CELL16_TYPE_BITS + 2))
+
 #define rep_SPECIAL_ENV   (rep_STRUCTURE(rep_structure)->special_env)
 
-#define rep_STRUCT_HASH(x,n) (((x) >> 4) % (n))
+#define rep_STRUCT_HASH(x,n) (((x) >> 3) % (n))
 
 
 /* binding tracking */
