@@ -1,6 +1,6 @@
 /* values.c -- Handling of Lisp data (includes garbage collection)
    Copyright (C) 1993, 1994 John Harper <john@dcs.warwick.ac.uk>
-   $Id: values.c,v 1.62 2000/07/14 11:08:51 john Exp $
+   $Id: values.c,v 1.65 2000/09/03 12:36:05 john Exp $
 
    This file is part of Jade.
 
@@ -487,7 +487,7 @@ rep_allocate_cons (void)
 }
 
 DEFUN("cons", Fcons, Scons, (repv car, repv cdr), rep_Subr2) /*
-::doc:cons::
+::doc:rep.data#cons::
 cons CAR CDR
 
 Returns a new cons-cell with car CAR and cdr CDR.
@@ -822,7 +822,8 @@ again:
     if(rep_INTP(val))
 	return;
 
-    if(rep_CONSP(val))
+    /* must be a cell */
+    if(rep_CELL_CONS_P(val))
     {
 	if(rep_CONS_WRITABLE_P(val))
 	{
@@ -925,7 +926,7 @@ again:
 }
 
 DEFUN("garbage-threshold", Fgarbage_threshold, Sgarbage_threshold, (repv val), rep_Subr1) /*
-::doc:garbage-threshold::
+::doc:rep.data#garbage-threshold::
 garbage-threshold [NEW-VALUE]
 
 The number of bytes of storage which must be used before a garbage-
@@ -936,7 +937,7 @@ collection is triggered.
 }
 
 DEFUN("idle-garbage-threshold", Fidle_garbage_threshold, Sidle_garbage_threshold, (repv val), rep_Subr1) /*
-::doc:idle-garbage-threshold::
+::doc:rep.data#idle-garbage-threshold::
 idle-garbage-threshold [NEW-VALUE]
 
 The number of bytes of storage which must be used before a garbage-
@@ -947,7 +948,7 @@ collection is triggered when the editor is idle.
 }
 
 DEFUN_INT("garbage-collect", Fgarbage_collect, Sgarbage_collect, (repv stats), rep_Subr1, "") /*
-::doc:garbage-collect::
+::doc:rep.data#garbage-collect::
 garbage-collect
 
 Scans all allocated storage for unusable data, and puts it onto the free-
@@ -1100,14 +1101,16 @@ rep_pre_values_init(void)
 void
 rep_values_init(void)
 {
+    repv tem = rep_push_structure ("rep.data");
     rep_ADD_SUBR(Scons);
     rep_ADD_SUBR(Sgarbage_threshold);
     rep_ADD_SUBR(Sidle_garbage_threshold);
     rep_ADD_SUBR_INT(Sgarbage_collect);
-    rep_ADD_SUBR(Smake_primitive_guardian);
-    rep_ADD_SUBR(Sprimitive_guardian_push);
-    rep_ADD_SUBR(Sprimitive_guardian_pop);
+    rep_ADD_INTERNAL_SUBR(Smake_primitive_guardian);
+    rep_ADD_INTERNAL_SUBR(Sprimitive_guardian_push);
+    rep_ADD_INTERNAL_SUBR(Sprimitive_guardian_pop);
     rep_INTERN_SPECIAL(after_gc_hook);
+    rep_pop_structure (tem);
 }
 
 void
@@ -1148,13 +1151,10 @@ rep_values_kill(void)
 
 /* Support for dumped Lisp code */
 
+#ifdef ENABLE_BROKEN_DUMPING
 void
 rep_dumped_init(char *file)
 {
-#if 1
-    fputs ("dumped startup is currently broken\n", stderr);
-    exit (1);
-#else
     void *dl = rep_open_dl_library (rep_string_dup (file));
     if (dl == 0)
 	fprintf (stderr, "warning: couldn't open dumped filed %s\n", file);
@@ -1178,5 +1178,5 @@ rep_dumped_init(char *file)
 		s->value = rep_void_value;
 	}
     }
-#endif
 }
+#endif
